// ===================================================================
// The Hologram Generator, version 2.0
// Copyright (c) 1998 by Zarniwoop (Jess Crable):
// ===================================================================
// 
// The Hologram Generator is an item that is currently using the model
// of the VoiceBox. You pick it up and activate it in a manner similar
// to the flare.  
//
// Once the Hologram Generator is on the ground, it will begin
// projecting a holographic image of yourself. The image will move
// through several animations according to what playertype you are.
// This image can serve as a decoy to distract anyone following you.
//
// I'm considering adding movement to this in future versions.
//
// You may adjust whether the holograms flicker or remain constant
// and whether or not the holograms will be solid or translucent.
// This can be accomplished by editing the default properties, under
// the category "HologramGenerator". 
// 
// Please report any bugs, problems or suggestions via e-mail to
// Zarniwoop@planetunreal.com 
// 
// Hit <TAB> during a game, and type:
// "summon Hologram.HologramGenerator" to get ahold of one! 
//
// This modification was inspired by the "Decoy" modification for
// Quake2 found at QDevels (http://www.planetquake.com/qdevels).
// Although I never used or looked at their code, I was definitely
// inspired by the concept.
//
// -------------------------------------------------------------------
// Files: Hologram.u   -> Put this in your Unreal/system directory.
// ===================================================================
// 
// Go to www.planetunreal.com/oddities for more small, unique Unreal
// modifications!
// 
// /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
// Legal:
// The Hologram Generator modification and classes contained within
// this modifcation are Copyright (c) 1998, Jess Crable (Zarniwoop).
// You can use the Hologram Generator modification in your own levels
// or modifications, and don't need permission from the author. You
// MUST give credit to the author, though, if you do use this
// modification.  
// 
// Also, do not modify this modification without permission from the
// author first. Finally, do not include this modification on any 
// CD-ROM without the author's permission.
// /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
// 
// ..Zarniwoop
//   http://www.planetunreal.com/oddities
//   zarniwoop@planetunreal.com


#exec TEXTURE IMPORT NAME=I_VoiceBox FILE=TEXTURES\HUD\i_voice.PCX GROUP="Icons" MIPS=OFF
#exec MESH IMPORT MESH=VoiceBoxMesh ANIVFILE=MODELS\voice_a.3D DATAFILE=MODELS\voice_d.3D X=0 Y=0 Z=0
#exec MESH ORIGIN MESH=VoiceBoxMesh X=0 Y=0 Z=-15 YAW=64
#exec MESH SEQUENCE MESH=VoiceBoxMesh SEQ=All   STARTFRAME=0  NUMFRAMES=10
#exec MESH SEQUENCE MESH=VoiceBoxMesh SEQ=Pulse STARTFRAME=0  NUMFRAMES=10
#exec TEXTURE IMPORT NAME=Ainv1 FILE=MODELS\inv.PCX GROUP="Skins"
#exec MESHMAP SCALE MESHMAP=VoiceBoxMesh X=0.04 Y=0.04 Z=0.08
#exec MESHMAP SETTEXTURE MESHMAP=VoiceBoxMesh NUM=1 TEXTURE=Ainv1
#exec AUDIO IMPORT FILE="Sounds\Hologram\STATIC.WAV" NAME="Static" GROUP="Hologram"

class HologramGenerator expands Pickup;

// This variable will represent the player throwing the generator.
// It's needed to find out what their mesh is, etc.
var PlayerPawn Projector;
// This vector is used for throwing the generator.
var vector X,Y,Z;
// Decision is used in a few different places in random situations.
var float Decision;
// This boolean variable determines whether the hologram will be
// translucent or not. It's defined in the default properties.
var() Bool TranslucentHolo;
// This boolean variable determines whether the hologram will be
// flickering or not. It's defined int he default properties.
var() Bool FlickeringOn;
// The duration variable is defined in default properties and is
// used to determine how long the generated hologram will last.
var() int Duration;

//var bool FadingIn;
// If flickering is turned on, the FullyProjected boolean variable
// will be used to produce a delay; when the hologram first begins
// to be projected it won't start flickering until it has successfully
// faded in once.
var bool FullyProjected;

// This state starts as soon as you activate the HologramGenerator
// in your inventory, usually with the <ENTER> key.

state Activated  
{

	// Lots of important variables are initialized in this function.
	// It's a good place to initialize things because it's executed
	// reliably when the state is entered.
	
	function BeginState()
	{
		// Set FullyProjected to False, since the hologram hasn't
		// been projected yet.
		
		FullyProjected = FALSE;
		
		// If it's going to be a translucent hologram (determined
		// in the default properties, defaulting to TRUE), set the
		// ScaleGlow property very low so it's basically invisible
		// to start.
		
		if (TranslucentHolo)
			ScaleGlow = 0.001;
		
		// Set the player holding the HologramGenerator to be the
		// "Projector".
		Projector = PlayerPawn(Owner);

		// A duration lower than 10 is silly, so check here to make
		// sure the default is above 10; if it's not, it will reset
		// it to be 10.
		
		if (Default.Duration < 10)
		Duration = 10;
		
		// Item's been activated.
		bActive = True;
	}
				
	// Once the HologramGenerator slows down enough it will stop
	// bouncing and become still, then it goes to the 'Playing'
	// state.

	function HitWall (vector HitNormal, actor Wall)
	{
		Velocity = 0.5*(( Velocity dot HitNormal ) * HitNormal * (-2.0) + Velocity);   
		If (VSize(Velocity) < 20)
		{
			bBounce = False;
			bStasis = false;
			SetPhysics(PHYS_None);
			GoToState('Playing');
		}
	}

// Below is the movement routine for when you first throw the
// HologramGenerator on the ground. Most of this came from the
// VoiceBox.

Begin:

	// Find where the player is facing and set the location.
	GetAxes(Pawn(Owner).ViewRotation,X,Y,Z);
	SetLocation(Pawn(Owner).Location+X*10+Y*8-Z*20);
	// Item shouldn't be touchable after it's thrown!
    Disable('Touch');
	bBounce=True;	
	// The Generator is thrown based on the player's current
	// velocity.
	Velocity = Owner.Velocity + Vector(Owner.Rotation) * 150.0;
	Velocity.z += 240;         
	DesiredRotation = RotRand();
	RotationRate.Yaw = 20000*FRand() - 10000;
	RespawnTime = 0.0; 
	
	// A hologram of the player is about to appear so you want to
	// adjust the item's collision attributes beforehand, or the
	// hologram may stick partially into the floor.
	SetCollisionSize(Projector.CollisionRadius, Projector.CollisionHeight);
	SetPhysics(PHYS_Falling);
	BecomePickup();
	bCollideWorld = true;

	// Adjusts the player's inventory settings and removes itself from
	// the inventory.
	Pawn(Owner).NextItem();
	if (Pawn(Owner).SelectedItem == Self) Pawn(Owner).SelectedItem=None;	

	Owner.PlaySound(ActivateSound);		
	Pawn(Owner).DeleteInventory(Self);	
}


state Playing
{
	// The HologramGenerator takes on many new attributes once it
	// begins projecting; once again the import settings are 
	// defined (or redefined) in the BeginState() function..
	
	Function BeginState()
	{
		// Set a new ambient sound, sort of a humming noise..
		AmbientSound = (sound'ffieldl2');
		// The player's hologram starts out tiny in size for a
		// neat effect when it starts, so set the drawscale to
		// tiny for now.
		DrawScale = 0.01;

//SetCollisionSize(Projector.Default.CollisionRadius, Projector.Default.CollisionHeight);
		// Redefine the mesh- it was once a VoiceBox, now it will
		// take on the same mesh as the Projector (former Owner).
		// Same deal with the skin and texture.
		Mesh = Projector.Mesh;
		Skin = Projector.Skin;
		Texture = Projector.Texture;

		// Set the hologram to translucent if it's been predefined
		// that way in the DefaultProperties.
		if (TranslucentHolo == True)
		{
		Style= STY_Translucent;
		}
		// Spawn a little energy burst for effect.
		Spawn(class'EnergyBurst',,,Location);		
	}

	// The tick function happens almost constantly, based on frames
	// rather than time. It's great to use if you need to
	// frequently check or update things.
	
	function Tick( float DeltaTime)
	{
		// This is a very rapid growing effect that happens when
		// the hologram is first projected (remember the drawscale
		// was set to 0.01 in the BeginState function). It'll keep
		// the hologram growing until it's the same size as the
		// player.
		
		If (DrawScale < Projector.Default.DrawScale)
		{
		DrawScale += 0.1;
		}

	// This is the flickering routine for a translucent hologram.
	// The flickering is very rapid.
	if (TranslucentHolo)
	{

		// If the ScaleGlow is less than 1 and the hologram hasn't
		// yet been completely projected (going from 0.001 to 1.0)
		// it won't start flickering. Below it will keep raising
		// the ScaleGlow in increments of 0.01; once it's at 1.0
		// it will set the FullyProjected variable to True, allowing
		// the hologram to begin flickering.
		
		if (ScaleGlow < 1 && !FullyProjected)
			{
			ScaleGlow += 0.01;
			if (ScaleGlow >= 1)
			{
				FullyProjected = TRUE;
			}
			}

		// The flickering routine is below.
		if (FlickeringOn)
		{
			if (FullyProjected)
			{

			// If the ScaleGlow is at 0.01, occasionally adjust the
			// ambientsound and reset the ScaleGlow to 1.0.
			if (ScaleGlow == 0.01)
			{
				if (FRand() < 0.3)
				AmbientSound = (sound'ffieldl2');		
				ScaleGlow = 1.0;
			}
			else
			// Call the flicker function (*sometimes*).
			if (FRand() < 0.4)
			{
				Flicker();
			}
			}			
		}
		}
}

	// Simple seperate function to be used in case the hologram is
	// not set to translucent, but flickering is on. The effect isn't
	// as good as if it were a translucent hologram, but it works.
	// Instead of using ScaleGlow, it turns the mesh completely off
	// and on, simulating flickering.
	
	function SolidFlicker()
	{
		Mesh = None;
		AmbientSound = (sound'Static');
	}
	
	// The flicker routine for a translucent hologram.
	Function Flicker()
	{
		if (TranslucentHolo)
		{
		// Reduce the ScaleGrow to 0.01 sometimes, making it appear
		// just about invisible.
		if (FRand() < 0.5)
		ScaleGlow = 0.01;
		}
	
	}
	
	// The restore function is used when the hologram isn't 
	// translucent and flickering is turned on. This restores
	// the hologram after it has disappeared during a flicker.
	Function Restore()
	{
		Mesh = Projector.Mesh;
		Skin = Projector.Skin;
		Texture = Projector.Texture;
	}


// The routines here are used in generating different animations
// for different playertypes (male, female, skaarj, etc..).

Begin:

FinishAnim();

// Keep executing the routine below as long as there is any duration
// left.

While (Duration > 0)
{
	// Make a decision.
	Decision = FRand();

	// If the player is using the SkaarjPlayer mesh, this hologram
	// is generated.
	if (Projector.IsA('SkaarjPlayer'))
	{
	if (Decision < 0.2)
	{
		FinishAnim();
		if (FRand() < 0.4)
		PlayAnim('Fighter');
		else
		PlayAnim('Lunge');
	}
	else
	if (Decision > 0.2 && Decision < 0.4)
	{
		FinishAnim();
		if (FRand() < 0.4)
		PlayAnim('CockGun');	
		else
		PlayAnim('Claw');
	}
	else
	if (Decision > 0.4 && Decision < 0.6)
	{
		FinishAnim();
		PlayAnim('Taunt1');
	}

	// If the decisions don't amount to anything, at least make
	// the hologram appear to be breathing.

	else
	{
	FinishAnim();
	LoopAnim('Breath');
	}
}

// Most of the player animations (aside from the SkaarjPlayer) are
// very similar, so if it's not a SkaarjPlayer it starts choosing
// what animations to use here instead.

else
{
	if (Decision < 0.2)
	{
		FinishAnim();
		if (FRand() < 0.4)
		PlayAnim('Taunt1');
		else
		{
		if (Projector.IsA('FemaleTwo'))
		PlayAnim('Victory1L');
		else
		PlayAnim('Victory1');
	}
	}
	else
	if (Decision > 0.2 && Decision < 0.31)
	{
		FinishAnim();
		PlayAnim('CockGun');	
	}
	else
	if (Decision > 0.32 && Decision < 0.6)
	{
		FinishAnim();
		PlayAnim('Taunt1L');
	}
		else
	{

	// If the decisions don't amount to anything, at least make
	// the hologram appear to be breathing.

	FinishAnim();
	LoopAnim('Breath1');
	}	
}
	// This starts the routine for a solid hologram with flickering
	// turned on; It will execute the SolidFlicker() function (which
	// turns the mesh off, rendering it invisible), sleep for a short
	// random duration, and then restore the mesh and reset the
	// ambientsound.
	
	if (FlickeringOn && !TranslucentHolo)
	{
		SolidFlicker();
		Sleep(FRand() - FRand() - FRand());
		Restore();
		AmbientSound = (sound'ffieldl2');		
	}

// Decrement the duration for the next loop.
Duration--;
}

// Duration has expired! Finish whatever animation was playing and
// prepare to destroy it in the Destroying state.
FinishAnim();
GotoState('Destroying');
}

// This state is entered when the HologramGenerator's duration has
// finally expired. Rather than just destroying the object it's nice
// to have some kind of an effect, so here it is.

state Destroying
{

	// Once again the Tick function is executed over and over very
	// rapidly.	
	function Tick( float DeltaTime)
	{
		// If it's translucent, make sure the ScaleGlow is 1.0 so
		// everyone can clearly see it before it destroys itself.
		if (TranslucentHolo)
		{
		if (ScaleGlow < 1.0)
		ScaleGlow = 1.0;
		}
		
		// Start making the hologram shrink- the idea is to make
		// it smoothly shrink to a tiny size, then have it disappear
		// in an energy burst followed with a sound.
		
		If (DrawScale > 0.1)
		{
			// Shrink!
			DrawScale -= 0.1;
		}
		
		// Once the DrawScale is less than 0.3, runt he DeProject()
		// function which will finish everything off.
		
		if (DrawScale < 0.3)
		{
			DeProject();
		}
	}
	
	function DeProject()
	{
		// Remove AmbientSound
		AmbientSound = None;

		// Spawn an energy burst at the hologram's current location.
		Spawn(class'EnergyBurst',,,Location);

		// Play a sound that sounds like it's shutting down.
		PlaySound(sound'dampndea');

		// Destroy it! It's over!
		Destroy();
	}    	

}

defaultproperties
{
     TranslucentHolo=True
     FlickeringOn=True
     duration=35
     bCanHaveMultipleCopies=True
     bActivatable=True
     bDisplayableInv=True
     PickupMessage="You got the Hologram Generator."
     PickupViewMesh=Mesh'UnrealI.VoiceBoxMesh'
     Charge=50
     ActivateSound=Sound'UnrealI.Pickups.FieldSnd'
     Icon=Texture'UnrealI.Icons.BlueSkull'
     Mesh=Mesh'UnrealI.VoiceBoxMesh'
     AmbientGlow=64
     SoundRadius=64
}
